import { Company, Identifiable, Internship, Mentor, Office } from "@/data";
import { momentSerializationTransformer, OneWayTransformer } from "@/serialization";
import { Nullable } from "@/helpers";
import { MentorDTO, mentorDtoTransformer } from "@/api/dto/mentor";
import { InternshipTypeDTO, internshipTypeDtoTransformer } from "@/api/dto/type";
import { Moment } from "moment";
import { sampleStudent } from "@/provider/dummy";

export enum SubmissionState {
    Draft,
    Submitted,
    Accepted,
    Rejected,
    Archival,
}

export interface InternshipRegistrationUpdateCompany {
    id: string,
    branchOffice: Identifiable,
}

export interface InternshipRegistrationUpdate {
    company: InternshipRegistrationUpdateCompany,
    start: string,
    end: string,
    type: number,
    mentor: MentorDTO,
}

export interface InternshipRegistrationDTO extends Identifiable {
    start: string;
    end: string;
    type: InternshipTypeDTO,
    state: SubmissionState,
    mentor: MentorDTO,
    company: Company,
    branchAddress: Office,
}

export interface InternshipInfoDTO {
    internshipRegistration: InternshipRegistrationDTO;
}

export const internshipRegistrationUpdateTransformer: OneWayTransformer<Nullable<Internship>, Nullable<InternshipRegistrationUpdate>> = {
    transform(subject: Nullable<Internship>, context?: unknown): Nullable<InternshipRegistrationUpdate> {
        return {
            start: subject?.startDate?.toISOString() || null,
            end: subject?.endDate?.toISOString() || null,
            type: parseInt(subject?.type?.id || "0"),
            mentor: mentorDtoTransformer.reverseTransform(subject.mentor as Mentor),
            company: {
                id: subject?.company?.id as string,
                branchOffice: {
                    id: subject?.office?.id
                },
            }
        }
    }
}

export const internshipRegistrationDtoTransformer: OneWayTransformer<InternshipRegistrationDTO, Internship> = {
    transform(dto: InternshipRegistrationDTO, context?: unknown): Internship {
        return {
            id: dto.id,
            office: dto.branchAddress,
            company: dto.company,
            mentor: mentorDtoTransformer.transform(dto.mentor),
            startDate: momentSerializationTransformer.reverseTransform(dto.start) as Moment,
            endDate: momentSerializationTransformer.reverseTransform(dto.end) as Moment,
            type: internshipTypeDtoTransformer.transform(dto.type),
            hours: 0,
            isAccepted: dto.state === SubmissionState.Accepted,
            lengthInWeeks: 0,
            program: [],
            intern: sampleStudent, // fixme
        };
    }
}
